import functools
import pickle


def save(fn):
    @functools.wraps(fn)
    def save_pickle(*args, **kwargs):
        try:
            with open('test_hashmap.pickle', 'rb') as f:
                loaded = pickle.load(f)
            return fn(loaded, *args, **kwargs)
        except FileNotFoundError:
            print('file not found')
        finally:
            with open('test_hashmap.pickle', 'wb') as f:
                pickle.dump(loaded, f)

    return save_pickle



import click
import pickle
import os
from nook.utils import save, HashMap

if os.path.isfile('hashmap.pickle'):
    pass
else:
    dictionary = HashMap()
    with open('hashmap.pickle', 'wb') as f:
        pickle.dump(dictionary, f)


@click.command(help='adds the key pair value')
@click.option('-k', '--key',help='key to add')
@click.option('-v', '--value',help='value to add')
@save
def add(loaded, key, value):
    loaded.set(key, value)
    click.echo(f'added key-pair \n[{key}]:[{value}]')


@click.command(help='deletes the key-pair value')
@click.option('-k', '--key',help ='requires the key to delete')
@save
def delete(loaded, key):
    loaded.remove_item(key)
    click.echo(f'deleted pair')


@click.command(help='retrieves the key-pair value')
@click.option('-k', '--key',help ='requires key')
@save
def getvalue(loaded, key):
    click.echo(loaded.get(key))


import unittest
from nook.utils import HashMap


class TestHashMap(unittest.TestCase):

    def setUp(self):
        self.hashmap = HashMap()

    def test_set_and_get(self):
        self.hashmap.set('key1', 'value1')
        self.assertEqual(self.hashmap.get('key1'), 'value1')

    def test_get_nonexistent_key(self):
        self.assertIsNone(self.hashmap.get('nonexistent'))

    def test_remove_item(self):
        self.hashmap.set('key1', 'value1')
        self.hashmap.remove_item('key1')
        self.assertIsNone(self.hashmap.get('key1'))

    def test_remove_nonexistent_item(self):
        self.hashmap.remove_item('nonexistent')  # Should not raise an exception


if __name__ == '__main__':
    unittest.main()



import unittest
from click.testing import CliRunner
from nook.main import cli
from nook.utils import HashMap
import pickle


class TestCLI(unittest.TestCase):

    def setUp(self):
        self.runner = CliRunner()

    def test_add_command(self):
        result = self.runner.invoke(cli, ['add', '--key', 'key1', '--value', 'value1'])
        self.assertIn('added key-pair', result.output)
        self.assertEqual(result.exit_code, 0)

        # Check if the key-value pair is stored
        with open('hashmap.pickle', 'rb') as f:
            hashmap = pickle.load(f)
            self.assertEqual(hashmap.get('key1'), 'value1')

    def test_delete_command(self):
        hashmap = HashMap()
        hashmap.set('key1', 'value1')
        with open('hashmap.pickle', 'wb') as f:
            pickle.dump(hashmap, f)

        result = self.runner.invoke(cli, ['delete', '--delete', 'key1'])
        self.assertIn('deleted pair', result.output)
        self.assertEqual(result.exit_code, 0)

        # Check if the key-value pair is removed
        with open('hashmap.pickle', 'rb') as f:
            hashmap = pickle.load(f)
            self.assertIsNone(hashmap.get('key1'))


if __name__ == '__main__':
    unittest.main()



import click
from nook import commands


@click.group()
@click.version_option("v0.1", prog_name="nook")
def cli():
    pass


cli.add_command(commands.add)
cli.add_command(commands.getvalue)
cli.add_command(commands.delete)


if __name__ == '__main__':
    cli()



import functools
import pickle


class HashMap:
    def __init__(self):
        self.array = [None for _ in range(100)]

    @staticmethod
    def __hash(key):
        count = 0
        for i in key:
            count += ord(i)
        return count

    def set(self, key, value):
        hashed = self.__hash(key) % len(self.array)
        if self.array[hashed] is None:
            self.array[hashed] = [(key, value)]
        else:

            for index, (k, v) in enumerate(self.array[hashed]):
                if k == key:
                    self.array[hashed][index] = (key, value)
                    return
            self.array[hashed].append((key, value))

    def get(self, key):
        hashed = self.__hash(key) % len(self.array)
        if self.array[hashed] is not None:
            for k, v in self.array[hashed]:
                if k == key:
                    return v
        return None

    def remove_item(self, key):
        hashed = self.__hash(key) % len(self.array)
        if self.array[hashed] is not None:
            if len(self.array[hashed]) == 1:
                self.array[hashed] = None
            else:
                for index, (k, v) in enumerate(self.array[hashed]):
                    if k == key:
                        self.array[hashed].pop(index)
                        return

    def __repr__(self):
        return f'{type(self)}'


def save(fn):
    @functools.wraps(fn)
    def save_pickle(*args, **kwargs):
        try:
            with open('hashmap.pickle', 'rb') as f:
                loaded = pickle.load(f)
            return fn(loaded, *args, **kwargs)
        except FileNotFoundError:
            print('file not found')
        finally:
            with open('hashmap.pickle', 'wb') as f:
                pickle.dump(loaded, f)

    return save_pickle